//------------------------------------------------------------------------------
// File: fvAddToRot.cpp
//
// Desc: Implementation of the fvAddToRot filter
//       
//
// Copyright (c) Vladimir Filippov. 01.01.2006.
//------------------------------------------------------------------------------

#include <streams.h>

#include <windows.h>
#include <math.h>
#include <initguid.h>
#include <olectl.h>

#if (1100 > _MSC_VER)
#include <olectlid.h>
#endif

#include "fvuids.h"
#include "ifvAddToRot.h"
#include "fvAddToRot.h"
#include "fvAddToRotProp.h"

#pragma warning(disable:4238)  // nonstd extension used: class rvalue used as lvalue


//   .
static const WCHAR g_wszName[] = L"fv Add Graph To ROT Sample Filter";

//        
const AMOVIESETUP_MEDIATYPE sudPinTypes =
{
    &MEDIATYPE_Video,       //   (,   ..)
    &MEDIASUBTYPE_NULL      //  ( ,    ..)
};

const AMOVIESETUP_PIN psudPins[] =
{
    {
        L"Input",           //   ( )
        FALSE,              // If TRUE, the filter renders the input from this pin. (Applies only to input pins. For output pins, the value is always FALSE)
        FALSE,              //  ?
        FALSE,              //     
        FALSE,              //      
        &CLSID_NULL,        //  
        L"Output",          //  
        1,                  // -     
        &sudPinTypes },     //     
      { L"Output",          //   ( )
        FALSE,              // If TRUE, the filter renders the input from this pin. (Applies only to input pins. For output pins, the value is always FALSE)
        TRUE,               //  ?
        FALSE,              //     
        FALSE,              //      
        &CLSID_NULL,        //  
        L"Input",           //  
        1,                  // -     
        &sudPinTypes        //     
    }
};

const AMOVIESETUP_FILTER sudfvAddToRotFilter =
{
    &CLSID_fvAddToRot,		//   (CLSID) 
    g_wszName,				//  
    0xA0000000,				//  merit
    2,						//  
    psudPins				//  
};


// List of class IDs and creator functions for the class factory. This
// provides the link between the OLE entry point in the DLL and an object
// being created. The class factory will call the static CreateInstance

CFactoryTemplate g_Templates[] = 
{
    { g_wszName
    , &CLSID_fvAddToRot				// !!!         ,
    , CfvAddToRot::CreateInstance	//        guidgen.exe
    , NULL
    , &sudfvAddToRotFilter }
,
    { L"fv Add To ROT Property Page"
    , &CLSID_fvAddToRotPropertyPage
    , CAddToRotProperties::CreateInstance }
};
int g_cTemplates = sizeof(g_Templates)/sizeof(g_Templates[0]);


/////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Constructor for CfvAddToRot
//
CfvAddToRot::CfvAddToRot(TCHAR *tszName,LPUNKNOWN punk,HRESULT *phr) :
    CTransInPlaceFilter(tszName, punk, CLSID_fvAddToRot, phr),
	CPersistStream(punk, phr),
	bParamChanged(FALSE),
	bIsActive(TRUE)
{

	//    
	HKEY hKey;
	if (RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\FVG Programs\\fvAddToRotFilter",0,KEY_READ,&hKey)==ERROR_SUCCESS) {
		unsigned long cbData=4;
		RegQueryValueEx(hKey,"bIsActive",0,NULL,(unsigned char*)&bIsActive,&cbData);
		RegCloseKey(hKey);
		bParamChanged=TRUE;
	}

	//    5    LOG_TRACE.
	DbgSetModuleLevel(LOG_TRACE,5);
} // CfvAddToRot


/////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Destructor for CfvAddToRot
//
CfvAddToRot::~CfvAddToRot()
{
	//   
	HKEY hKey;
	unsigned long dwDisposition;
	if (bParamChanged) {
		if (RegCreateKeyEx(HKEY_CURRENT_USER,"Software\\FVG Programs\\fvAddToRotFilter",0,NULL,REG_OPTION_NON_VOLATILE,KEY_WRITE,NULL,&hKey,&dwDisposition)==ERROR_SUCCESS) {
			RegSetValueEx(hKey,"bIsActive",0,REG_DWORD,(unsigned char*)&bIsActive,4);
			RegCloseKey(hKey);
		}
	}
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////
// CreateInstance
//
// Provide the way for COM to create a CfvAddToRot object
//
CUnknown * WINAPI CfvAddToRot::CreateInstance(LPUNKNOWN punk, HRESULT *phr) 
{
    ASSERT(phr);
    
	//   . ,   ,     
	// .    NAME(),      NULL.
    CfvAddToRot *pNewObject = new CfvAddToRot(NAME("fv Add Graph To ROT Sample Filter"), punk, phr);
    if (pNewObject == NULL) {
        if (phr)
            *phr = E_OUTOFMEMORY;
    }

    return pNewObject;
} // CreateInstance
/////////////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////////////
// NonDelegatingQueryInterface
//
//
STDMETHODIMP CfvAddToRot::NonDelegatingQueryInterface(REFIID riid, void **ppv)
{
    CheckPointer(ppv,E_POINTER);

    if (riid == IID_IfvAddToRot) {
        
		return GetInterface((IfvAddToRot *) this, ppv);

    } else if (riid == IID_ISpecifyPropertyPages) {
        
		return GetInterface((ISpecifyPropertyPages *) this, ppv);

    } else if (riid == IID_IPersistStream) {

		return GetInterface((IPersistStream *) this, ppv);

	} else {

        return CTransformFilter::NonDelegatingQueryInterface(riid, ppv);
    }

} // NonDelegatingQueryInterface 
/////////////////////////////////////////////////////////////////////////////////////////////////////////

//   .
//  S_OK    , 
// E_INVALIDARG   .
//         
HRESULT CfvAddToRot::CheckInputType(const CMediaType* mtIn)
{
	CheckPointer(mtIn,E_POINTER);

	//      
	// ..     5   LOG_TRACE,
	//        .
	DisplayType("Input Type",mtIn);

	return E_INVALIDARG;
}


//      
//   bIsActive,   TRUE,    ROT
STDMETHODIMP CfvAddToRot::JoinFilterGraph(IFilterGraph *pGraph, LPCWSTR pName)
{
	DWORD dwRot;
	CAutoLock cAutoLock(&m_fvLock);
	if (bIsActive) AddToRot(pGraph,&dwRot);
	return CBaseFilter::JoinFilterGraph(pGraph,pName);
}

//     ROT
//
HRESULT CfvAddToRot::AddToRot(IUnknown *pUnkGraph, DWORD *pdwRegister) 
{
	//      .   
	// " ",      ROT
	HANDLE g_hEvent=CreateEvent(NULL,TRUE,FALSE,"fvAddToRotEvent001");
	if (WaitForSingleObject(g_hEvent,0)==WAIT_OBJECT_0) return S_OK;
	SetEvent(g_hEvent);

	DbgLog((LOG_TRACE,5,"Adding graph to ROT"));

    IMoniker * pMoniker;
    IRunningObjectTable *pROT;
    if (FAILED(GetRunningObjectTable(0, &pROT))) {
        return E_FAIL;
    }
    WCHAR wsz[256];
    wsprintfW(wsz, L"FilterGraph %08x pid %08x", (DWORD_PTR)pUnkGraph, GetCurrentProcessId());
    HRESULT hr = CreateItemMoniker(L"!", wsz, &pMoniker);
    if (SUCCEEDED(hr)) {
        hr = pROT->Register(ROTFLAGS_REGISTRATIONKEEPSALIVE, pUnkGraph,
            pMoniker, pdwRegister);
        pMoniker->Release();
    }
    pROT->Release();

    return hr;
}

//
// Transform
//
//        .
//      ,   
//  (     CTransformFilter
//      ).
//       .
HRESULT CfvAddToRot::Transform(IMediaSample *pMediaSample)
{
	return NOERROR;
} // Transform


//
// GetPages
//
//   ISpecifyPropertyPages
//       CLSID  .
//
STDMETHODIMP CfvAddToRot::GetPages(CAUUID *pPages)
{
    CheckPointer(pPages,E_POINTER);

    pPages->cElems = 1;
    pPages->pElems = (GUID*)CoTaskMemAlloc(sizeof(GUID));
    if (pPages->pElems == NULL) return E_OUTOFMEMORY;

    *(pPages->pElems) = CLSID_fvAddToRotPropertyPage;
    return NOERROR;

} // GetPages


//   IfvAddToRot
//     .
STDMETHODIMP CfvAddToRot::get_Active(int *bActive)
{
	CAutoLock cAutoLock(&m_fvLock);
	CheckPointer(bActive,E_POINTER);
	*bActive=bIsActive;
	return NOERROR;
}

//   IfvAddToRot
//     .
STDMETHODIMP CfvAddToRot::put_Active(int bActive)
{
	CAutoLock cAutoLock(&m_fvLock);
	bIsActive=bActive;
	bParamChanged=TRUE;
	SetDirty(TRUE);
	return NOERROR;
}



//===  CPersistStream

STDMETHODIMP CfvAddToRot::GetClassID(CLSID *pClsid)
{
    CheckPointer(pClsid,E_POINTER);
    *pClsid = CLSID_fvAddToRot;
    return NOERROR;

} // GetClassID

//
// SizeMax
//
// Override CPersistStream method.
//  . ,    
//
int CfvAddToRot::SizeMax()
{
    //     Unicode.  
    //       24 
    //
    return 1*24;

}  // SizeMax


//
// WriteToStream
//
// Override CPersistStream method.
// Write our properties to the stream.
//
HRESULT CfvAddToRot::WriteToStream(IStream *pStream)
{
    HRESULT hr;

    hr = WriteInt(pStream, bIsActive);
    if (FAILED(hr)) return hr;

    return NOERROR;
} // WriteToStream


//
// ReadFromStream
//
// Override CPersistStream method.
// Read our properties from the stream.
//
HRESULT CfvAddToRot::ReadFromStream(IStream *pStream)
{
    HRESULT hr;

    bIsActive = ReadInt(pStream, hr);
    if (FAILED(hr)) return hr;

    return NOERROR;
} // ReadFromStream



////////////////////////////////////////////////////////////////////////
//
// Exported entry points for registration and unregistration 
// (in this case they only call through to default implementations).
//
////////////////////////////////////////////////////////////////////////

//
// DllRegisterServer
//
// Handle registration of this filter
//
STDAPI DllRegisterServer()
{
    return AMovieDllRegisterServer2(TRUE);

} // DllRegisterServer


//
// DllUnregisterServer
//
STDAPI DllUnregisterServer()
{
    return AMovieDllRegisterServer2(FALSE);

} // DllUnregisterServer


//
// DllEntryPoint
//
extern "C" BOOL WINAPI DllEntryPoint(HINSTANCE, ULONG, LPVOID);

//
BOOL APIENTRY DllMain(HANDLE hModule, DWORD  dwReason, LPVOID lpReserved)
{
	return DllEntryPoint((HINSTANCE)(hModule), dwReason, lpReserved);
}

