//------------------------------------------------------------------------------
// File: fvAddToRotProp.cpp
//
// Desc: Implementation of property page class
//       for the fvAddToRot filter.
//
// Copyright (c) Vladimir Filippov. 2005.
//------------------------------------------------------------------------------

#include <stdio.h>
#include <streams.h>
#include <commctrl.h>
#include <olectl.h>
#include <memory.h>
#include "resource.h"
#include "fvuids.h"
#include "ifvAddToRot.h"
#include "fvAddToRot.h"
#include "fvAddToRotProp.h"


//
// CreateInstance
//
// This goes in the factory template table to create new filter instances
//
CUnknown * WINAPI CAddToRotProperties::CreateInstance(LPUNKNOWN lpunk, HRESULT *phr)
{
    ASSERT(phr);

    CUnknown *punk = new CAddToRotProperties(lpunk, phr);

    if(punk == NULL) {
        if (phr)
            *phr = E_OUTOFMEMORY;
    }

    return punk;

} // CreateInstance


//
// Constructor
//
CAddToRotProperties::CAddToRotProperties(LPUNKNOWN pUnk, HRESULT *phr) :
    CBasePropertyPage(NAME("AddToRot Filter Property Page"), pUnk,
                      IDD_ATRDLG, IDS_TITLE),
    m_pfvAddToRotCfg(NULL),
	bIsActive(TRUE)
{
    InitCommonControls();	

} // (Constructor)


//
// SetDirty
//
//   m_bDirty      , 
//    (   "").
//
void CAddToRotProperties::SetDirty()
{
    m_bDirty = TRUE;

    if (m_pPageSite) {
        m_pPageSite->OnStatusChange(PROPPAGESTATUS_DIRTY);
    }

} // SetDirty


//
// OnReceiveMessage
//
//       
// .
INT_PTR CAddToRotProperties::OnReceiveMessage(HWND hwnd,
                                           UINT uMsg,
                                           WPARAM wParam,
                                           LPARAM lParam)
{

    switch(uMsg)
    {
        case WM_INITDIALOG:
        {
            return (LRESULT) 1;
        }

        case WM_VSCROLL:
        {
            return (LRESULT) 1;
        }

        case WM_COMMAND:
        {
            if(LOWORD(wParam) == IDC_ISACTIVE)
            {
				bIsActive=IsDlgButtonChecked(m_hwnd,IDC_ISACTIVE);
                SetDirty();
				return (LRESULT) 1;
            }
			
		}		

        case WM_DESTROY:
        {
            return (LRESULT) 1;
        }
    }

    return CBasePropertyPage::OnReceiveMessage(hwnd,uMsg,wParam,lParam);

} // OnReceiveMessage


//     
HRESULT CAddToRotProperties::OnActivate()
{
	if (m_pfvAddToRotCfg) m_pfvAddToRotCfg->get_Active(&bIsActive);
    CheckDlgButton(m_hwnd, IDC_ISACTIVE, bIsActive);
	        
    return NOERROR;
}


//
// OnConnect
//
//       .
//
HRESULT CAddToRotProperties::OnConnect(IUnknown *pUnknown)
{
    ASSERT(m_pfvAddToRotCfg == NULL);
    CheckPointer(pUnknown,E_POINTER);

    HRESULT hr = pUnknown->QueryInterface(IID_IfvAddToRot, (void **)&m_pfvAddToRotCfg);
    if (FAILED(hr)) return E_NOINTERFACE;

    ASSERT(m_pfvAddToRotCfg);

    return NOERROR;

} // OnConnect


//
// OnDisconnect
//
//     .
//
HRESULT CAddToRotProperties::OnDisconnect()
{
    //   
    if (m_pfvAddToRotCfg) {
		m_pfvAddToRotCfg->Release();
		m_pfvAddToRotCfg = NULL;
	}

    return NOERROR;

} // OnDisconnect


//
// OnDeactivate
//
//   .
//
HRESULT CAddToRotProperties::OnDeactivate()
{
    return NOERROR;
} // OnDeactivate


//
// OnApplyChanges
//
//    .
//
HRESULT CAddToRotProperties::OnApplyChanges()
{
	if (m_pfvAddToRotCfg) return m_pfvAddToRotCfg->put_Active(bIsActive);
    
	return E_UNEXPECTED;
} // OnApplyChanges



